// CONFIGURATION //
$Playtime::Config::AutoSaveInterval = 15; // in minutes, if 0 or below then autosave will be disabled
$Playtime::Config::SaveDirectory = "config/server/playtime/"; // must include trailing forward slash
$Playtime::Config::LoadingDoesNotCount = true; // if true then users' playtimes will not be recorded while in loading screen
///////////////////

function mAdd(%a, %b) { // evading scientific notation
	%c = %a + %b;
	return %c | 0;
}

function playtimeFormatConversion() { // this function will be removed in the future
	%completed = $Playtime::Config::SaveDirectory @ "_converted.txt";
	
	if (isFile(%completed))
		return;
		
	warn("Starting playtime format conversion...");
	
	%find = $Playtime::Config::SaveDirectory @ "*.txt";
	
	for (%i = findFirstFile(%find); %i !$= ""; %i = findNextFile(%find)) {
		if (isFile(%i) || fileName(%i) !$= "_converted.txt") {
			%read = new fileObject();
			%read.openForRead(%i);
			%convert = %read.readLine();
			%read.close();
			%read.delete();
			
			if (getWord(%convert, 3) $= "") {
				%time = getWord(%convert, 0) / 1000;
				
				%date = getWords(%convert, 1, 2);
				
				%days = mFloor(%time / 86400);
				%hours = mFloor((%time % 86400) / 3600);
				%minutes = mFloor(((%time % 86400) % 3600) / 60);
				%seconds = ((%time % 86400) % 3600) % 60;
				
				%convert = %days SPC %hours SPC %minutes SPC %seconds SPC %date;
				
				%write = new fileObject();
				%write.openForWrite(%i);
				%write.writeLine(%convert);
				%write.close();
				%write.delete();
			}
		}
	}
	
	%file = new fileObject();
	%file.openForWrite(%completed);
	%file.writeLine("don't delete");
	%file.close();
	%file.delete();
	
	warn("Playtime format conversion complete!");
}

playtimeFormatConversion();

function gameConnection::incPlaytime(%client) {
	%playtime = %client.playtime["session"];
	%days = getWord(%playtime, 0);
	%hours = getWord(%playtime, 1);
	%minutes = getWord(%playtime, 2);
	%seconds = getWord(%playtime, 3);
	
	%seconds = mAdd(%seconds, 1);
	
	if (%seconds > 59) {
		%seconds = 0;
		%minutes = mAdd(%minutes, 1);
	}
	
	if (%minutes > 59) {
		%minutes = 0;
		%hours = mAdd(%hours, 1);
	}
	
	if (%hours > 23) {
		%hours = 0;
		%days = mAdd(%days, 1);
	}
	
	%client.playtime["session"] = %days SPC %hours SPC %minutes SPC %seconds;
	
	%playtime = %client.playtime["total"];
	%days = getWord(%playtime, 0);
	%hours = getWord(%playtime, 1);
	%minutes = getWord(%playtime, 2);
	%seconds = getWord(%playtime, 3);
	
	%seconds = mAdd(%seconds, 1);

	if (%seconds > 59) {
		%seconds = 0;
		%minutes = mAdd(%minutes, 1);
	}
	
	if (%minutes > 59) {
		%minutes = 0;
		%hours = mAdd(%hours, 1);
	}
	
	if (%hours > 23) {
		%hours = 0;
		%days = mAdd(%days, 1);
	}
	
	%client.playtime["total"] = %days SPC %hours SPC %minutes SPC %seconds;
} 

function gameConnection::getSessionPlaytime(%client) {
	return %client.playtime["session"];
}

function gameConnection::getTotalPlaytime(%client) {
	return %client.playtime["total"];
}

function gameConnection::getLastPlayed(%client) {
	return %client.playtime["lastplayed"];
}

function gameConnection::savePlaytime(%client) {
	%file = new fileObject();
	%file.openForWrite($Playtime::Config::SaveDirectory @ %client.bl_id @ ".txt");
	%file.writeLine(%client.playtime["total"] SPC getDateTime());
	%file.close();
	%file.delete();
}

function gameConnection::loadPlaytime(%client) {
	%data = $Playtime::Config::SaveDirectory @ %client.bl_id @ ".txt";

	if (!isFile(%data))
		return;

	%file = new fileObject();
	%file.openForRead(%data);
	%playtime = %file.readLine();
	%file.close();
	%file.delete();
	
	%client.playtime["total"] = getWords(%playtime, 0, 3);
	%client.playtime["lastplayed"] = getWords(%playtime, 4, 5);
}

function serverCmdPlaytime(%client, %a1, %a2, %a3) {
	if (%a1 !$= "") {
		%clObj = findClientByName(trim(%a1 SPC %a2 SPC %a3));
		if (!%clObj)
			return;
	} else {
		%clObj = %client;
	}
	
	messageClient(%client, '', "\c3" @ %clObj.name @ "\c0's Playtime Record");
	
	%playtime = %clObj.getTotalPlaytime();
	%days = getWord(%playtime, 0);
	%hours = getWord(%playtime, 1);
	%minutes = getWord(%playtime, 2);
	%seconds = getWord(%playtime, 3);
	
	messageClient(%client, '', " \c6Total: " @ %days @ "\c1d\c6:" @ %hours @ "\c1h\c6:" @ %minutes @ "\c1m\c6:" @ %seconds @ "\c1s");
	
	%playtime = %clObj.getSessionPlaytime();
	%days = getWord(%playtime, 0);
	%hours = getWord(%playtime, 1);
	%minutes = getWord(%playtime, 2);
	%seconds = getWord(%playtime, 3);
	
	messageClient(%client, '', " \c6Session: " @ %days @ "\c1d\c6:" @ %hours @ "\c1h\c6:" @ %minutes @ "\c1m\c6:" @ %seconds @ "\c1s");
	
	if (%clObj.playtime["lastplayed"] !$= "")
		messageClient(%client, '', " \c6Last Played: \c4" @ %clObj.playtime["lastplayed"]);
	else
		messageClient(%client, '', " \c6Last Played: \c4Never");
}

function playtimeAutoSaveTick() {
	cancel($Playtime::AutoSaveTock);
	
	if ($Playtime::Config::AutoSaveInterval > 0) {
		for (%i = 0; %i < clientGroup.getCount(); %i++) {
			%clObj = clientGroup.getObject(%i);
			if (%clObj.hasSpawnedOnce && $Playtime::Config::LoadingDoesNotCount) {
				%clObj.savePlaytime();
			} else if (!$Playtime::Config::LoadingDoesNotCount) {
				%clObj.savePlaytime();
			}
		}
	}
	
	$Playtime::AutoSaveTock = schedule($Playtime::Config::AutoSaveInterval * 60000, 0, playtimeAutoSaveTick);
}

cancel($Playtime::AutoSaveTock);
$Playtime::AutoSaveTock = schedule($Playtime::Config::AutoSaveInterval * 60000, 0, playtimeAutoSaveTick);

function playtimeTick() {
	cancel($Playtime::Tock);	
	
	for (%i = 0; %i < clientGroup.getCount(); %i++) {
		%clObj = clientGroup.getObject(%i);
		if (%clObj.hasSpawnedOnce && $Playtime::Config::LoadingDoesNotCount) {
			%clObj.incPlaytime();
		} else if (!$Playtime::Config::LoadingDoesNotCount) {
			%clObj.incPlaytime();
		}
	}
	
	$Playtime::Tock = schedule(1000 * getTimeScale(), 0, playtimeTick);
}

playtimeTick();

if (isPackage(playtime))
	deactivatePackage(playtime);

package playtime {
	
	function gameConnection::autoAdminCheck(%client) {
		%client.playtime["total"] = 0 SPC 0 SPC 0 SPC 0;
		%client.playtime["session"] = 0 SPC 0 SPC 0 SPC 0;
		%client.playtime["lastplayed"] = "";
		
		%client.loadPlaytime();
		
		return parent::autoAdminCheck(%client);
	}
	
	function gameConnection::onClientLeaveGame(%client) {
		%client.savePlaytime();
		
		return parent::onClientLeaveGame(%client);
	}
	
};

activatePackage(playtime);